#version 120

#define COLORED_SHADOWS 1 //0: Sombras normales, 1: Sombras de color, 2: Sin sombras
#define SHADOW_BRIGHTNESS 0.75 // Nivel de brillo en sombras

uniform sampler2D lightmap;
uniform sampler2D shadowcolor0;
uniform sampler2D shadowtex0;
uniform sampler2D shadowtex1;
uniform sampler2D texture;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec4 shadowPos;

void main() {
    vec4 color = texture2D(texture, texcoord) * glcolor;
    vec2 lm = lmcoord;
    
    // Verifica si la superficie está orientada hacia la fuente de luz
    if (shadowPos.w > 0.0) {
        bool inShadow = false;

        #if COLORED_SHADOWS == 0
            inShadow = texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z;
        #else
            inShadow = texture2D(shadowtex1, shadowPos.xy).r < shadowPos.z;
        #endif

        if (inShadow) {
            lm.y *= SHADOW_BRIGHTNESS;
        } else {
            // Superficie en luz directa
            lm.y = mix(31.0 / 32.0 * SHADOW_BRIGHTNESS, 31.0 / 32.0, sqrt(shadowPos.w));

            #if COLORED_SHADOWS == 1
                if (texture2D(shadowtex0, shadowPos.xy).r < shadowPos.z) {
                    // Aplicar color de sombra cuando hay objetos translúcidos en la trayectoria de la luz
                    vec4 shadowLightColor = texture2D(shadowcolor0, shadowPos.xy);
                    shadowLightColor.rgb = mix(vec3(1.0), shadowLightColor.rgb, shadowLightColor.a);
                    shadowLightColor.rgb = mix(shadowLightColor.rgb, vec3(1.0), lm.x);
                    color.rgb *= shadowLightColor.rgb;
                }
            #endif
        }
    }

    // Multiplicar por el mapa de iluminación
    color *= texture2D(lightmap, lm);

    /* DRAWBUFFERS:0 */
    gl_FragData[0] = color; // Salida final
}
