#version 120

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;
uniform vec3 shadowLightPosition;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec4 shadowPos;

#include "/distort.glsl"

void main() {
    // Cálculo de coordenadas de textura y color
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord  = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    glcolor = gl_Color;

    // Calcular la posición en el espacio de vista
    vec4 viewPos = gl_ModelViewMatrix * gl_Vertex;
    vec3 normalizedNormal = normalize(gl_NormalMatrix * gl_Normal);
    float lightDot = dot(normalize(shadowLightPosition), normalizedNormal);

    if (lightDot > 0.0) { // Superficie orientada hacia la luz
        vec4 playerPos = gbufferModelViewInverse * viewPos;
        shadowPos = shadowProjection * (shadowModelView * playerPos); // Convertir a espacio de sombras

        // Aplicar sesgo y distorsión a shadowPos
        float bias = computeBias(shadowPos.xyz);
        shadowPos.xyz = distort(shadowPos.xyz) * 0.5 + 0.5;

        #ifdef NORMAL_BIAS
            vec3 normalProj = mat3(shadowModelView) * (mat3(gbufferModelViewInverse) * normalizedNormal);
            shadowPos.xyz += (shadowProjection[0][0] * normalProj) * bias;
        #else
            shadowPos.z -= bias / abs(lightDot);
        #endif
    } else { 
        lmcoord.y *= SHADOW_BRIGHTNESS; // Sombra garantizada en el lado opuesto
        shadowPos = vec4(0.0); // No es necesario verificar el mapa de sombras
    }

    shadowPos.w = lightDot;
    gl_Position = gl_ProjectionMatrix * viewPos; // Posición final
}
