#version 120

uniform mat4 gbufferModelView;
uniform mat4 gbufferModelViewInverse;
uniform mat4 shadowModelView;
uniform mat4 shadowProjection;
uniform vec3 shadowLightPosition;

varying vec2 lmcoord;
varying vec2 texcoord;
varying vec4 glcolor;
varying vec3 shadowPos;

#include "/distort.glsl"

void main() {
    // Optimizar cálculo de coordenadas de textura
    texcoord = (gl_TextureMatrix[0] * gl_MultiTexCoord0).xy;
    lmcoord = (gl_TextureMatrix[1] * gl_MultiTexCoord1).xy;
    glcolor = gl_Color;

    // Calcular posición en el espacio de vista
    vec4 viewPos = gl_ModelViewMatrix * gl_Vertex;
    vec4 playerPos = gbufferModelViewInverse * viewPos;

    // Convertir a espacio NDC (Normalized Device Coordinates)
    shadowPos = (shadowProjection * shadowModelView * playerPos).xyz;

    // Aplicar distorsión y ajustar al espacio de pantalla de sombra
    shadowPos = distort(shadowPos) * 0.5 + 0.5;
    
    // Calcular y aplicar shadow bias solo una vez
    shadowPos.z -= computeBias(shadowPos);

    // Posición final del vértice
    gl_Position = gl_ProjectionMatrix * viewPos;
}
